(*
 * A symbol table.
 *
 * ----------------------------------------------------------------
 *
 * Copyright (C) 1999 Jason Hickey, Caltech
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Jason Hickey
 * jyh@cs.caltech.edu
 *)

(*
 * Representation of symbols.
 *)
type symbol

(*
 * Add a symbol to the table.
 *)
val add : string -> symbol

(*
 * Adding for debugging.
 * Don't use this normally.
 *)
val debug_add : string -> symbol

(*
 * Make a new symbol.
 *)
val new_symbol : symbol -> symbol
val new_symbol_string : string -> symbol

(*
 * Get back the string.
 *)
val to_string : symbol -> string

(*
 * Printer uses Format library.
 *)
val print_symbol : symbol -> unit

(*
 * Compare two symbols for equality.
 *)
val eq : symbol -> symbol -> bool

(*
 * Ordered comparisons of symbols.
 *)
val compare : symbol -> symbol -> int

(*
 * This table provides associations between symbols
 * and values.
 *)
module SymbolSet : Set_sig.SetSig with type elt = symbol
module SymbolTable : Set_sig.TableSig with type elt = symbol

(*
 * -*-
 * Local Variables:
 * Caml-master: "set"
 * End:
 * -*-
 *)
